// C Header File
// Created 01/01/03; 12:31:21

#ifndef __MACROS_H__
#define __MACROS_H__


// to avoid the warper only needed for compatibility with AMS1
#define custom_EV_getAppID _rom_call(AppID,(const char *),454)

// DialogAddText, but the second argument is the flags for DialogAdd ()
#define DialogAddTextAndFlags(h,f,x,y,t) DialogAdd(h,f,x,y,7,(const char*)(t))
// DialogAddPullDown, but the second argument is the flags for DialogAdd ()
#define DialogAddPulldownAndFlags(h,f,x,y,t,p,i) DialogAdd(h,f,x,y,14, \
	(const char*)(t),(short)(p),(short)(i))

// ROM Calls dealing with Flash Apps installation :
// void PrepareAppsMem (BOOL DoUnreloc, BOOL ShowStatusBar);
// trap #11 - d3.l = 0x7 - Unrelocation of all the Flash Apps
#define PrepareAppsMem _rom_call(void,(BOOL,BOOL),416)
// BOOL InstallApp (void *AppPtr);
// trap #11 - d3.l = 0xE - App installation and relocation
// Returns FALSE if there was not enough memory to create the ACB.
// The ROM Call has no entry in the ROM Calls table :(
#define InstallApp(AppPtr) \
	({BOOL result; \
	  asm ("move.l %1,%%d4;" \
	       "moveq #1,%%d5;" /* (BOOL)TRUE - DoReloc */ \
	       "moveq #0xE,%%d3; trap #11;" \
	       "move.l %%d0,%0" \
        : "=g" (result) : "g" (AppPtr) \
        : "a0", "a1", "d0", "d1", "d2", "d3", "d4", "d5", "d7" ); \
    result;})
// This ROM Call scans the linked list of the ACBs, searches the cleared flags
// ACB_INSTALLED and ACB_BUILTIN, sends CM_INSTALL to the App, sets
// ACB_JT_VERSION if the App is an APP_BACKGROUND (see Attr 1), calls the
// method OO_APP_NOTICE_INSTALL of all the other Apps, and sets ACB_INSTALLED.
#define SendInstallMsgs _rom_call(void,(void),428)


#define fatal(s) do {ST_helpMsg(s); return;} while (0)
#define CST_C(val89,val92) (TI89 ? val89 : val92)

#define PRGM_NAME "Flash Apps Hider"
#define PRGM_VERSION "v0.1"


// to create the apps data :
#define NO_MORE_APPS_ID 0xFFF0u   // used by PrepareAppsData()
#define NEW_APP_ID 0x8100u   // used by PrepareAppsData()
#define APP_HEADER_MAGIC 0x167B533Dl   // at the begin of an app header

// to draw the menu :
#define APP_NAME_MAX_LEN  CST_C (20,32)  // max lenghth supported by the AMS
#define MENU_W  CST_C (130,202)   // width
#define MENU_COORD_X  CST_C(10,11)
#define MENU_COORD_X_NO_SCROLLING  (MENU_COORD_X-7)// when there is no scrolling
// number of pulldowns to show without scrolling
#define MENU_MAX_PULLDOWNS  (unsigned short)(CST_C (5,6))
#define MENU_BOTTOM  CST_C (13,16) // height reserved for the buttons of the dialog box
// printed before the name if the App is hidden
#define HIDDEN_SYMBOL_STR  CST_C ("\x0A4  ","\x0A4 ")
#define NO_H_SYMBOL_STR  CST_C ("     ","  ")
// y coordinate of the first pulldown
#define PULLDOWN_Y_ORIGIN  (MENU_TEXT_Y + CST_C (10,16))
#define PULLDOWN_Y_STEP  CST_C (9,12)   // pixels between each pulldown
#define MENU_TITLE CST_C (PRGM_NAME" "PRGM_VERSION, \
	PRGM_NAME" "PRGM_VERSION" : select the Apps to hide/unhide")
#define MEM_MENU_TITLE  PRGM_NAME" "PRGM_VERSION
#define MENU_TEXT_89 "Select the Apps to hide/unhide :"
#define MENU_TEXT_X 5   // for TI89
#define MENU_TEXT_Y CST_C (15,0)


// for debugging purposes :
#define DB_msg(s) {ST_helpMsg(s);}
#define DB_msg_pause(s) {DB_msg(s); ngetchx(); ST_eraseHelp();}
#define DB_msg_break(s) {DB_msg(s); DB_break();}
#define DB_break() {asm ("0: bra.s 0b");}
#define DB_pause() {ngetchx();}
#define DB_mark(s) {asm ("/* MARK : "#s" */");}


#endif     // #ifndef __MACROS_H__
